package sun.management;

import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.OpenType;
import java.lang.management.MemoryUsage;
import com.sun.management.GcInfo;
import javax.management.openmbean.CompositeType;
import java.lang.management.GarbageCollectorMXBean;

public class GcInfoBuilder
{
    private final GarbageCollectorMXBean gc;
    private final String[] poolNames;
    private String[] allItemNames;
    private CompositeType gcInfoCompositeType;
    private final int gcExtItemCount;
    private final String[] gcExtItemNames;
    private final String[] gcExtItemDescs;
    private final char[] gcExtItemTypes;
    
    GcInfoBuilder(GarbageCollectorMXBean gc, String[] poolNames) {
        this.gc = gc;
        this.poolNames = poolNames;
        this.gcExtItemCount = this.getNumGcExtAttributes(gc);
        this.gcExtItemNames = new String[this.gcExtItemCount];
        this.gcExtItemDescs = new String[this.gcExtItemCount];
        this.gcExtItemTypes = new char[this.gcExtItemCount];
        this.fillGcAttributeInfo(gc, this.gcExtItemCount, this.gcExtItemNames, this.gcExtItemTypes, this.gcExtItemDescs);
        this.gcInfoCompositeType = null;
    }
    
    GcInfo getLastGcInfo() {
        return this.getLastGcInfo0(this.gc, this.gcExtItemCount, new Object[this.gcExtItemCount], this.gcExtItemTypes, new MemoryUsage[this.poolNames.length], new MemoryUsage[this.poolNames.length]);
    }
    
    public String[] getPoolNames() {
        return this.poolNames;
    }
    
    int getGcExtItemCount() {
        return this.gcExtItemCount;
    }
    
    synchronized CompositeType getGcInfoCompositeType() {
        if (this.gcInfoCompositeType != null) {
            return this.gcInfoCompositeType;
        }
        String[] baseGcInfoItemNames = GcInfoCompositeData.getBaseGcInfoItemNames();
        OpenType[] baseGcInfoItemTypes = GcInfoCompositeData.getBaseGcInfoItemTypes();
        int length = baseGcInfoItemNames.length;
        int n = length + this.gcExtItemCount;
        this.allItemNames = new String[n];
        String[] itemDescriptions = new String[n];
        OpenType[] itemTypes = new OpenType[n];
        System.arraycopy(baseGcInfoItemNames, 0, this.allItemNames, 0, length);
        System.arraycopy(baseGcInfoItemNames, 0, itemDescriptions, 0, length);
        System.arraycopy(baseGcInfoItemTypes, 0, itemTypes, 0, length);
        if (this.gcExtItemCount > 0) {
            this.fillGcAttributeInfo(this.gc, this.gcExtItemCount, this.gcExtItemNames, this.gcExtItemTypes, this.gcExtItemDescs);
            System.arraycopy(this.gcExtItemNames, 0, this.allItemNames, length, this.gcExtItemCount);
            System.arraycopy(this.gcExtItemDescs, 0, itemDescriptions, length, this.gcExtItemCount);
            for (int i = 0; i < this.gcExtItemCount; ++length, ++i) {
                switch (this.gcExtItemTypes[i]) {
                    case 'Z': {
                        itemTypes[length] = SimpleType.BOOLEAN;
                        break;
                    }
                    case 'B': {
                        itemTypes[length] = SimpleType.BYTE;
                        break;
                    }
                    case 'C': {
                        itemTypes[length] = SimpleType.CHARACTER;
                        break;
                    }
                    case 'S': {
                        itemTypes[length] = SimpleType.SHORT;
                        break;
                    }
                    case 'I': {
                        itemTypes[length] = SimpleType.INTEGER;
                        break;
                    }
                    case 'J': {
                        itemTypes[length] = SimpleType.LONG;
                        break;
                    }
                    case 'F': {
                        itemTypes[length] = SimpleType.FLOAT;
                        break;
                    }
                    case 'D': {
                        itemTypes[length] = SimpleType.DOUBLE;
                        break;
                    }
                    default: {
                        throw new AssertionError("Unsupported type [" + this.gcExtItemTypes[length] + "]");
                    }
                }
            }
        }
        CompositeType gcInfoCompositeType;
        try {
            gcInfoCompositeType = new CompositeType("sun.management." + this.gc.getName() + ".GcInfoCompositeType", "CompositeType for GC info for " + this.gc.getName(), this.allItemNames, itemDescriptions, itemTypes);
        }
        catch (OpenDataException ex) {
            throw Util.newException(ex);
        }
        return this.gcInfoCompositeType = gcInfoCompositeType;
    }
    
    synchronized String[] getItemNames() {
        if (this.allItemNames == null) {
            this.getGcInfoCompositeType();
        }
        return this.allItemNames;
    }
    
    private native int getNumGcExtAttributes(GarbageCollectorMXBean p0);
    
    private native void fillGcAttributeInfo(GarbageCollectorMXBean p0, int p1, String[] p2, char[] p3, String[] p4);
    
    private native GcInfo getLastGcInfo0(GarbageCollectorMXBean p0, int p1, Object[] p2, char[] p3, MemoryUsage[] p4, MemoryUsage[] p5);
}
